@@ -1,7 +1,9 @@
 # -*- coding: utf-8 -*-
-# pylint: disable=line-too-long
-# pylint: disable=invalid-name
 # pylint: disable=bare-except
+# pylint: disable=too-many-branches
+# pylint: disable=too-many-lines
+# pylint: disable=too-many-statements
+# pylint disable=invalid-name
 
 """
 Copyright (c) 2019-2021 Tomasz Łuczak, TeaM-TL
@@ -45,7 +47,7 @@ import os
 import platform
 import sys
 import tempfile
-from PIL import Image, ImageGrab
+from PIL import ImageGrab
 
 # my modules
 import convert
@@ -71,16 +73,12 @@ if mswindows.windows() == 1:
 localedir = os.path.join(os.path.abspath(os.path.dirname(__file__)), 'locale')
 if not os.path.isdir(localedir):
     localedir = os.path.join(os.getcwd(), 'locale')
-translate_info = str("Locale directory: " + localedir)
-log.write_log(translate_info, "M")
+log.write_log(str("Locale directory: " + localedir), "M")
 
 translate = gettext.translation('fotokilof', localedir, fallback=True)
 gettext.install('fotokilof', localedir)
 _ = translate.gettext
 
-translate_info = str(gettext.find('base', 'locales'))
-log.write_log(translate_info, "M")
-
 ###################
 # CONSTANTS
 if mswindows.windows() == 1:
@@ -92,7 +90,8 @@ else:
     PREVIEW_NEW = 450
     PREVIEW_LOGO = 100
 
-preview_size_list = (300, 350, 400, 450, 500, 550, 600, 650, 700, 800, 900, 1000, 1200, 1400, 1600, 1800, 1920, 'none')
+preview_size_list = (300, 350, 400, 450, 500, 550, 600, 650, 700, 800,
+        900, 1000, 1200, 1400, 1600, 1800, 1920, 'none')
 ##########################
 
 
@@ -253,6 +252,10 @@ def apply_all_convert(out_file, write_command):
         previous_command = 1
         cmd = cmd + " " + convert.convert_rotate(img_rotate.get())
 
+    if img_mirror_on.get() > 0:
+        previous_command = 1
+        cmd = cmd + " " + convert.convert_mirror(img_mirror_flip.get(),img_mirror_flop.get())
+
     if img_border_on.get() == 1:
         previous_command = 1
         border = int(e_border.get())
@@ -449,6 +452,22 @@ def convert_rotate_button():
     progress_files.set(_("done"))
 
 
+def convert_mirror_button():
+    """ mirror button """
+    progress_files.set(_("Processing"))
+    root.update_idletasks()
+    out_file = magick.pre_magick(file_in_path.get(),
+                                 work_dir.get(),
+                                 co_apply_type.get())
+    cmd = convert.convert_mirror(img_mirror_flip.get(), img_mirror_flop.get())
+    cmd_magick = GM_or_IM + "convert"
+    print_command(cmd)
+    result = magick.magick(cmd, file_in_path.get(), out_file, cmd_magick)
+    if result == "OK":
+        preview_new(out_file)
+    progress_files.set(_("done"))
+
+
 def convert_resize_button():
     """ Resize button """
     progress_files.set(_("Processing"))
@@ -572,10 +591,8 @@ def convert_text_entries():
     dict_return['text'] = e_text.get()
     dict_return['dx'] = e_text_x.get()
     dict_return['dy'] = e_text_y.get()
-    if img_text_gravity_onoff.get():
-        dict_return['gravitation'] = img_text_gravity.get()
-    else:
-        dict_return['gravitation'] = "0"
+    dict_return['gravitation'] = img_text_gravity.get()
+    dict_return['gravitation_onoff'] = img_text_gravity_onoff.get()
     if mswindows.windows() == 1:
         dict_return['font'] = img_text_font_dict[img_text_font.get()]
     else:
@@ -645,7 +662,8 @@ def open_file_logo():
                      (_("SVG files"), ".SVG"),
                      (_("ALL types"), "*"))
     else:
-        filetypes = ((_("All graphics files"), ".JPG .jpg .JPEG .jpeg .PNG .png .TIF .tif .TIFF .tiff"),
+        filetypes = ((_("All graphics files"),
+                    ".JPG .jpg .JPEG .jpeg .PNG .png .TIF .tif .TIFF .tiff"),
                      (_("JPG files"), ".JPG .jpg .JPEG .jpeg"),
                      (_("PNG files"), ".PNG .png"),
                      (_("TIFF files"), ".TIF .tif .TIFF .tiff"),
@@ -672,7 +690,8 @@ def open_file():
                      (_("SVG files"), ".SVG"),
                      (_("ALL types"), "*"))
     else:
-        filetypes = ((_("All graphics files"), ".JPG .jpg .JPEG .jpeg .PNG .png .TIF .tif .TIFF .tiff"),
+        filetypes = ((_("All graphics files"),
+                    ".JPG .jpg .JPEG .jpeg .PNG .png .TIF .tif .TIFF .tiff"),
                      (_("JPG files"), ".JPG .jpg .JPEG .jpeg"),
                      (_("PNG files"), ".PNG .png"),
                      (_("TIFF files"), ".TIF .tif .TIFF .tiff"),
@@ -686,9 +705,9 @@ def open_file():
         extension_from_file()
         file_in_path.set(result)
         root.title(window_title + file_in_path.get())
-        image = Image.open(file_in_path.get())
-        file_in_width.set(image.width)
-        file_in_height.set(image.height)
+        image_size =  magick.get_image_size(file_in_path.get())
+        file_in_width.set(image_size[0])
+        file_in_height.set(image_size[1])
         preview_orig()
     else:
         preview_orig_clear()
@@ -713,6 +732,9 @@ def open_file_last():
                 try:
                     file_in_path.set(os.path.normpath(os.path.join(cwd, filename)))
                     root.title(window_title + file_in_path.get())
+                    image_size =  magick.get_image_size(file_in_path.get())
+                    file_in_width.set(image_size[0])
+                    file_in_height.set(image_size[1])
                     preview_orig()
                     extension_from_file()
                     preview_new_refresh("none")
@@ -739,6 +761,9 @@ def open_file_next():
                 try:
                     file_in_path.set(os.path.normpath(os.path.join(cwd, filename)))
                     root.title(window_title + file_in_path.get())
+                    image_size =  magick.get_image_size(file_in_path.get())
+                    file_in_width.set(image_size[0])
+                    file_in_height.set(image_size[1])
                     preview_orig()
                     extension_from_file()
                 except:
@@ -768,6 +793,9 @@ def open_file_first():
                 try:
                     file_in_path.set(os.path.normpath(os.path.join(cwd, filename)))
                     root.title(window_title + file_in_path.get())
+                    image_size =  magick.get_image_size(file_in_path.get())
+                    file_in_width.set(image_size[0])
+                    file_in_height.set(image_size[1])
                     preview_orig()
                     extension_from_file()
                     preview_new_refresh("none")
@@ -794,16 +822,15 @@ def open_file_prev():
                 try:
                     file_in_path.set(os.path.normpath(os.path.join(cwd, filename)))
                     root.title(window_title + file_in_path.get())
+                    image_size =  magick.get_image_size(file_in_path.get())
+                    file_in_width.set(image_size[0])
+                    file_in_height.set(image_size[1])
                     preview_orig()
+                    extension_from_file()
+                    preview_new_refresh("none")
                 except:
                     log.write_log("Error in open_file_first", "E")
 
-                directory = os.path.dirname(file_in_path.get())
-                file_in_path.set(os.path.join(directory, filename))
-                preview_orig()
-                extension_from_file()
-                preview_new_refresh("none")
-
 
 def open_screenshot():
     """ Make screenshot """
@@ -988,6 +1015,11 @@ def ini_read_wraper():
     e_logo_dy.delete(0, "end")
     e_logo_dy.insert(0, ini_entries['logo_dy'])
 
+    ini_entries = ini_read.ini_read_mirror(FILE_INI)
+    img_mirror_on.set(ini_entries['img_mirror_on'])
+    img_mirror_flip.set(ini_entries['img_mirror_flip'])
+    img_mirror_flop.set(ini_entries['img_mirror_flop'])
+
 
 def ini_save():
     """ Write variables into config file INI """
@@ -1058,6 +1090,10 @@ def ini_save():
     config.set('Contrast', 'selection', co_contrast_selection.get())
     config.set('Contrast', 'contrast_stretch_1', e1_contrast.get())
     config.set('Contrast', 'contrast_stretch_2', e2_contrast.get())
+    config.add_section('Mirror')
+    config.set('Mirror', 'on', str(img_mirror_on.get()))
+    config.set('Mirror', 'flip', str(img_mirror_flip.get()))
+    config.set('Mirror', 'flop', str(img_mirror_flop.get()))
     config.add_section('Logo')
     config.set('Logo', 'on', str(img_logo_on.get()))
     config.set('Logo', 'logo', file_logo_path.get())
@@ -1082,12 +1118,10 @@ def help_info(event):
     # global PWD
     try:
         license_file = os.path.join(PWD, "LICENSE")
-        file = open(license_file, "r", encoding="utf8")
-
-        message = ""
-        for i in file:
-            message = message + i
-        # file.close
+        with open(license_file, "r", encoding="utf8") as licensef:
+            message = ""
+            for i in licensef:
+                message = message + i
     except:
         log.write_log("help_info: error during loading license file", "W")
         message = "Copyright " + version.__copyright__ + " " + version.__author__ + " under MIT license"
@@ -1121,11 +1155,16 @@ def mouse_crop_NW(event):
     height = int(y_preview*xy_max['y_orig']/xy_max['y_max'])
 
     if img_crop_on.get() == 1:
-        e1_crop_1.delete(0, "end")
-        e1_crop_1.insert(0, width)
-        e2_crop_1.delete(0, "end")
-        e2_crop_1.insert(0, height)
-
+        if img_crop.get() == 1:
+            e1_crop_1.delete(0, "end")
+            e1_crop_1.insert(0, width)
+            e2_crop_1.delete(0, "end")
+            e2_crop_1.insert(0, height)
+        elif img_crop.get() == 2:
+            e1_crop_2.delete(0, "end")
+            e1_crop_2.insert(0, width)
+            e2_crop_2.delete(0, "end")
+            e2_crop_2.insert(0, height)
         preview_orig()
 
     if img_text_on.get() and not img_text_gravity_onoff.get():
@@ -1169,7 +1208,7 @@ def preview_orig():
             if img_crop_on.get() == 1:
                 # draw crop rectangle on preview
                 xy_max = common.mouse_crop_calculation(file_in_width.get(),
-                                               file_in_width.get(),
+                                               file_in_height.get(),
                                                int(co_preview_selector_orig.get()))
                 if img_crop.get() == 1:
                     x0 = int(e1_crop_1.get())
@@ -1236,7 +1275,9 @@ def preview_orig():
                 log.write_log("preview_orig: Cannot load image size", "E")
 
             if img_histograms_on.get() == 1:
-                pi_histogram_orig.configure(file=preview.preview_histogram(file_in_path.get(), GM_or_IM))
+                pi_histogram_orig.configure(
+                        file=preview.preview_histogram(file_in_path.get(),
+                        GM_or_IM))
                 l_histogram_orig.configure(image=pi_histogram_orig)
         else:
             preview_orig_clear()
@@ -1339,6 +1380,11 @@ def tools_set(preview_on):
     else:
         frame_normalize.grid()
 
+    if img_mirror_on.get() == 0:
+        frame_mirror.grid_remove()
+    else:
+        frame_mirror.grid()
+
     if img_logo_on.get() == 0:
         frame_logo.grid_remove()
     else:
@@ -1509,11 +1555,16 @@ img_border_on = IntVar()  # Border
 img_border_color = StringVar()
 img_normalize_on = IntVar()  # Normalize
 img_normalize = IntVar()  #  (1,2,3)
-normalize_channels = ("None", "Red", "Green", "Blue", "Alpha", "Gray", "Cyan", "Magenta", "Yellow", "Black", "Opacity", "Index", "RGB", "RGBA", "CMYK", "CMYKA")
+normalize_channels = ("None", "Red", "Green", "Blue", "Alpha", "Gray",
+                      "Cyan", "Magenta", "Yellow", "Black", "Opacity",
+                      "Index", "RGB", "RGBA", "CMYK", "CMYKA")
 img_bw_on = IntVar()  # Black-white
 img_bw = IntVar()
 img_contrast_on = IntVar()  # Contrast
 img_contrast = IntVar()  # (1, 2)
+img_mirror_on = IntVar()  # Mirror
+img_mirror_flip = IntVar()  # (0, 1)
+img_mirror_flop = IntVar()  # (0, 1)
 contrast_selection = ("+3", "+2", "+1", "0", "-1", "-2", "-3")
 img_custom_on = IntVar()  # Custom
 progress_var = IntVar()  # progressbar
@@ -1657,6 +1708,10 @@ cb_contrast = ttk.Checkbutton(frame_tools_set, text=_("Contrast"),
                               variable=img_contrast_on,
                               offvalue="0", onvalue="1",
                               command=tools_set_off)
+cb_mirror = ttk.Checkbutton(frame_tools_set, text=_("Mirror"),
+                              variable=img_mirror_on,
+                              offvalue="0", onvalue="1",
+                              command=tools_set_off)
 cb_logo = ttk.Checkbutton(frame_tools_set, text=_("Logo"),
                           variable=img_logo_on,
                           offvalue="0", onvalue="1",
@@ -1683,6 +1738,7 @@ cb_border.pack(padx=5, pady=1, anchor=W, side='left')
 cb_bw.pack(padx=5, pady=1, anchor=W, side='left')
 cb_contrast.pack(padx=5, pady=1, anchor=W, side='left')
 cb_normalize.pack(padx=5, pady=1, anchor=W, side='left')
+cb_mirror.pack(padx=5, pady=1, anchor=W, side='left')
 cb_logo.pack(padx=5, pady=1, anchor=W, side='left')
 cb_custom.pack(padx=5, pady=1, anchor=W, side='left')
 cb_histograms.pack(padx=5, pady=1, anchor=W, side='left')
@@ -1954,7 +2010,7 @@ b_text_run.grid(row=5, column=5, sticky=(E), padx=5, pady=5)
 ###########################
 # Rotate
 ###########################
-frame_rotate = ttk.Labelframe(frame_first_col, text=_("Rotate"),
+frame_rotate = ttk.LabelFrame(frame_first_col, text=_("Rotate"),
                               style="Fiolet.TLabelframe")
 frame_rotate.grid(row=5, column=1, sticky=(N, W, E, S), padx=5, pady=1)
 ###
@@ -2019,14 +2075,14 @@ rb2_bw.grid(row=1, column=1, padx=5, pady=5, sticky=W)
 e_bw_sepia.grid(row=1, column=2, padx=5, pady=5, sticky=E)
 l_bw_sepia.grid(row=1, column=3, padx=5, pady=5, sticky=W)
 rb1_bw.grid(row=2, column=1, padx=5, pady=0, sticky=W)
-b_bw_run.grid(row=3, column=2, columnspan=2, padx=5, pady=5, sticky=E)
+b_bw_run.grid(row=2, column=2, columnspan=2, padx=5, pady=5, sticky=E)
 
 ########################
 # Contrast
 #########################
 frame_contrast = ttk.Labelframe(frame_first_col, text=_("Contrast"),
                                 style="Fiolet.TLabelframe")
-frame_contrast.grid(row=7, column=1, sticky=(N, W, E, S), padx=5, pady=1)
+frame_contrast.grid(row=7, column=1, rowspan=2, sticky=(N, W, E, S), padx=5, pady=1)
 ###
 rb1_contrast = ttk.Radiobutton(frame_contrast, text=_("Stretch"),
                                variable=img_contrast, value="1")
@@ -2081,14 +2137,36 @@ rb1_normalize.grid(row=1, column=1, padx=5, pady=0, sticky=W)
 l_normalize_channel.grid(row=1, column=2, padx=5, pady=4, sticky=E)
 co_normalize_channel.grid(row=1, column=3, padx=5, pady=4, sticky=E)
 rb2_normalize.grid(row=2, column=1, padx=5, pady=4, sticky=W)
-b_normalize_run.grid(row=3, column=2, columnspan=2, padx=5, pady=4, sticky=E)
+b_normalize_run.grid(row=2, column=2, columnspan=2, padx=5, pady=4, sticky=E)
+
+
+###########################
+# Mirror
+###########################
+frame_mirror = ttk.LabelFrame(frame_first_col, text=_("Mirror"),
+                              style="Fiolet.TLabelframe")
+frame_mirror.grid(row=8, column=2, sticky=(N, E, S), padx=5, pady=1)
+
+cb_mirror_flip = ttk.Checkbutton(frame_mirror, text=_("Flip"),
+                                 variable=img_mirror_flip,
+                                 offvalue="0", onvalue="1")
+cb_mirror_flop = ttk.Checkbutton(frame_mirror, text=_("Flop"),
+                                 variable=img_mirror_flop,
+                                 offvalue="0", onvalue="1")
+b_mirror_run = ttk.Button(frame_mirror, text=_("Execute"),
+                          style="Brown.TButton",
+                          command=convert_mirror_button)
+
+cb_mirror_flip.grid(row=1, column=1, sticky=(N, W, E, S), padx=5, pady=5)
+cb_mirror_flop.grid(row=1, column=2, sticky=(N, W, E, S), padx=5, pady=5)
+b_mirror_run.grid(row=1, column=5, sticky=E, padx=5, pady=5)
 
 ###########################
 # Logo
 ###########################
-frame_logo = ttk.Labelframe(frame_first_col, text=_("Logo"),
+frame_logo = ttk.LabelFrame(frame_first_col, text=_("Logo"),
                             style="Fiolet.TLabelframe")
-frame_logo.grid(row=10, column=1, columnspan=2,
+frame_logo.grid(row=11, column=1, columnspan=2,
                 sticky=(N, W, E, S), padx=5, pady=1)
 
 b_logo_select = ttk.Button(frame_logo, text=_("File selection"),
@@ -2308,9 +2386,9 @@ if GM_or_IM is not None:
     l_border.configure(bg=img_border_color.get())
     if os.path.isfile(file_in_path.get()):
         root.title(window_title + file_in_path.get())
-        image_size =  magick.get_image_size(file_in_path.get())
-        file_in_width.set(image_size[0])
-        file_in_height.set(image_size[1])
+        image_dimension =  magick.get_image_size(file_in_path.get())
+        file_in_width.set(image_dimension[0])
+        file_in_height.set(image_dimension[1])
         crop_tool_hide_show()
     if img_logo_on.get() == 1:
         if os.path.isfile(file_logo_path.get()):
